@@ -24,7 +24,6 @@
 #include <QString>
 #include <QTemporaryDir>
 #include <QTemporaryFile>
-#include <QWindow>
 
 #include <KIO/FileCopyJob>
 #include <KIO/ListJob>
@@ -33,13 +32,12 @@
 #include <KRecentDocument>
 #include <KSharedConfig>
 #include <KSystemClipboard>
-#include <KWindowSystem>
 
 ExportManager::ExportManager(QObject *parent)
     : QObject(parent)
-    , mImageSavedNotInTemp(false)
-    , mSavePixmap(QPixmap())
-    , mTempFile(QUrl())
+    , m_imageSavedNotInTemp(false)
+    , m_savePixmap(QPixmap())
+    , m_tempFile(QUrl())
 {
     connect(this, &ExportManager::imageSaved, &Settings::setLastSaveLocation);
     connect(this, &ExportManager::imageSavedAndCopied, &Settings::setLastSaveLocation);
@@ -47,7 +45,7 @@ ExportManager::ExportManager(QObject *parent)
 
 ExportManager::~ExportManager()
 {
-    delete mTempDir;
+    delete m_tempDir;
 }
 
 ExportManager *ExportManager::instance()
@@ -62,76 +60,76 @@ ExportManager *ExportManager::instance()
 
 QPixmap ExportManager::pixmap() const
 {
-    return mSavePixmap;
+    return m_savePixmap;
 }
 
 void ExportManager::setWindowTitle(const QString &windowTitle)
 {
-    mWindowTitle = windowTitle;
+    m_windowTitle = windowTitle;
 }
 
 QString ExportManager::windowTitle() const
 {
-    return mWindowTitle;
-}
-
-Spectacle::CaptureMode ExportManager::captureMode() const
-{
-    return mCaptureMode;
-}
-
-void ExportManager::setCaptureMode(Spectacle::CaptureMode theCaptureMode)
-{
-    mCaptureMode = theCaptureMode;
+    return m_windowTitle;
 }
 
 void ExportManager::setPixmap(const QPixmap &pixmap)
 {
-    mSavePixmap = pixmap;
+    m_savePixmap = pixmap;
 
     // reset our saved tempfile
-    if (mTempFile.isValid()) {
-        mUsedTempFileNames.append(mTempFile);
-        QFile file(mTempFile.toLocalFile());
+    if (m_tempFile.isValid()) {
+        m_usedTempFileNames.append(m_tempFile);
+        QFile file(m_tempFile.toLocalFile());
         file.remove();
-        mTempFile = QUrl();
+        m_tempFile = QUrl();
     }
 
     // since the pixmap was modified, we now consider the image unsaved
-    mImageSavedNotInTemp = false;
+    m_imageSavedNotInTemp = false;
+
+    Q_EMIT pixmapChanged();
 }
 
 void ExportManager::updatePixmapTimestamp()
 {
-    mPixmapTimestamp = QDateTime::currentDateTime();
+    m_pixmapTimestamp = QDateTime::currentDateTime();
 }
 
 void ExportManager::setTimestamp(const QDateTime &timestamp)
 {
-    mPixmapTimestamp = timestamp;
+    m_pixmapTimestamp = timestamp;
 }
 
 // native file save helpers
 
-QString ExportManager::defaultSaveLocation() const
+static QString ensureDefaultLocationExists(const QUrl &saveUrl)
 {
-    const QUrl saveUrl = Settings::self()->defaultSaveLocation();
-    QString savePath = saveUrl.scheme().isEmpty() ? saveUrl.toString() : saveUrl.toLocalFile();
+    QString savePath = saveUrl.isRelative() ? saveUrl.toString() : saveUrl.toLocalFile();
     savePath = QDir::cleanPath(savePath);
 
     QDir savePathDir(savePath);
     if (!(savePathDir.exists())) {
         savePathDir.mkpath(QStringLiteral("."));
     }
-
     return savePath;
 }
 
-QUrl ExportManager::getAutosaveFilename()
+QString ExportManager::defaultSaveLocation() const
+{
+    return ensureDefaultLocationExists(Settings::self()->defaultSaveLocation());
+}
+
+QString ExportManager::defaultVideoSaveLocation() const
+{
+    return ensureDefaultLocationExists(Settings::self()->defaultVideoSaveLocation());
+}
+
+QUrl ExportManager::getAutosaveFilename() const
 {
     const QString baseDir = defaultSaveLocation();
     const QDir baseDirPath(baseDir);
-    const QString filename = makeAutosaveFilename();
+    const QString filename = formattedFilename();
     const QString fullpath =
         autoIncrementFilename(baseDirPath.filePath(filename), Settings::self()->defaultSaveImageFormat().toLower(), &ExportManager::isFileExists);
 
@@ -143,7 +141,22 @@ QUrl ExportManager::getAutosaveFilename()
     }
 }
 
-QString ExportManager::truncatedFilename(QString const &filename)
+QString ExportManager::suggestedVideoFilename(const QString &extension) const
+{
+    const QString baseDir = defaultVideoSaveLocation();
+    const QDir baseDirPath(baseDir);
+    const QString filename = formattedFilename(Settings::self()->saveVideoFormat());
+    const QString fullpath = autoIncrementFilename(baseDirPath.filePath(filename), extension, &ExportManager::isFileExists);
+
+    const QUrl fileNameUrl = QUrl::fromUserInput(fullpath);
+    if (fileNameUrl.isValid() && fileNameUrl.isLocalFile()) {
+        return fileNameUrl.toLocalFile();
+    } else {
+        return {};
+    }
+}
+
+QString ExportManager::truncatedFilename(QString const &filename) const
 {
     QString result = filename;
     constexpr auto maxFilenameLength = 255;
@@ -154,21 +167,15 @@ QString ExportManager::truncatedFilename(QString const &filename)
     return result;
 }
 
-QString ExportManager::makeAutosaveFilename()
+QString ExportManager::formattedFilename(const QString &nameTemplate) const
 {
-    return formatFilename(Settings::self()->saveFilenameFormat());
-}
-
-QString ExportManager::formatFilename(const QString &nameTemplate)
-{
-    const QDateTime timestamp = mPixmapTimestamp;
+    const QDateTime timestamp = m_pixmapTimestamp;
     QString baseName = nameTemplate;
     QString baseDir = defaultSaveLocation();
-    QString title;
+    QString title = m_windowTitle;
 
-    if (mCaptureMode == Spectacle::CaptureMode::ActiveWindow || mCaptureMode == Spectacle::CaptureMode::TransientWithParent
-        || mCaptureMode == Spectacle::CaptureMode::WindowUnderCursor) {
-        title = mWindowTitle.replace(QLatin1Char('/'), QLatin1String("_")); // POSIX doesn't allow "/" in filenames
+    if (!title.isEmpty()) {
+        title.replace(QLatin1Char('/'), QLatin1String("_")); // POSIX doesn't allow "/" in filenames
     } else {
         // Remove '%T' with separators around it
         const auto wordSymbol = QStringLiteral(R"(\p{L}\p{M}\p{N})");
@@ -178,14 +185,14 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     }
 
     QString result = baseName.replace(QLatin1String("%Y"), timestamp.toString(QStringLiteral("yyyy")))
-                         .replace(QLatin1String("%y"), timestamp.toString(QStringLiteral("yy")))
-                         .replace(QLatin1String("%M"), timestamp.toString(QStringLiteral("MM")))
-                         .replace(QLatin1String("%D"), timestamp.toString(QStringLiteral("dd")))
-                         .replace(QLatin1String("%H"), timestamp.toString(QStringLiteral("hh")))
-                         .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
-                         .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")))
-                         .replace(QLatin1String("%t"), timestamp.toString(QStringLiteral("t")))
-                         .replace(QLatin1String("%T"), title);
+                             .replace(QLatin1String("%y"), timestamp.toString(QStringLiteral("yy")))
+                             .replace(QLatin1String("%M"), timestamp.toString(QStringLiteral("MM")))
+                             .replace(QLatin1String("%D"), timestamp.toString(QStringLiteral("dd")))
+                             .replace(QLatin1String("%H"), timestamp.toString(QStringLiteral("hh")))
+                             .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
+                             .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")))
+                             .replace(QLatin1String("%t"), timestamp.toString(QStringLiteral("t")))
+                             .replace(QLatin1String("%T"), title);
 
     // check if basename includes %[N]d token for sequential file numbering
     QRegularExpression paddingRE;
@@ -260,7 +267,7 @@ QString ExportManager::formatFilename(const QString &nameTemplate)
     return truncatedFilename(result);
 }
 
-QString ExportManager::autoIncrementFilename(const QString &baseName, const QString &extension, FileNameAlreadyUsedCheck isFileNameUsed)
+QString ExportManager::autoIncrementFilename(const QString &baseName, const QString &extension, FileNameAlreadyUsedCheck isFileNameUsed) const
 {
     QString result = truncatedFilename(baseName) + QLatin1String(".") + extension;
     if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
@@ -282,7 +289,7 @@ QString ExportManager::autoIncrementFilename(const QString &baseName, const QStr
     return truncatedFilename(result) + extension;
 }
 
-QString ExportManager::makeSaveMimetype(const QUrl &url)
+QString ExportManager::imageFileSuffix(const QUrl &url) const
 {
     QMimeDatabase mimedb;
     const QString type = mimedb.mimeTypeForUrl(url).preferredSuffix();
@@ -293,9 +300,12 @@ QString ExportManager::makeSaveMimetype(const QUrl &url)
     return type;
 }
 
-bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
+bool ExportManager::writeImage(QIODevice *device, const QByteArray &suffix)
 {
-    QImageWriter imageWriter(device, format);
+    // In the documentation for QImageWriter, it is a bit ambiguous what "format" means.
+    // From looking at how QImageWriter handles the built-in supported formats internally,
+    // "format" basically means the file extension, not the mimetype.
+    QImageWriter imageWriter(device, suffix);
     imageWriter.setQuality(Settings::self()->compressionQuality());
     /** Set compression 50 if the format is png. Otherwise if no compression value is specified
      *  it will fallback to using quality (QTBUG-43618) and produce huge files.
@@ -303,7 +313,7 @@ bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
      *  enabled by default and only disabled if compression is set to 0, also any value except 0
      *  has the same effect for them.
      */
-    if (format == "png") {
+    if (suffix == "png") {
         imageWriter.setCompression(50);
     }
     if (!(imageWriter.canWrite())) {
@@ -311,10 +321,10 @@ bool ExportManager::writeImage(QIODevice *device, const QByteArray &format)
         return false;
     }
 
-    return imageWriter.write(mSavePixmap.toImage());
+    return imageWriter.write(m_savePixmap.toImage());
 }
 
-bool ExportManager::localSave(const QUrl &url, const QString &mimetype)
+bool ExportManager::localSave(const QUrl &url, const QString &suffix)
 {
     // Create save directory if it doesn't exist
     const QUrl dirPath(url.adjusted(QUrl::RemoveFilename));
@@ -331,14 +341,14 @@ bool ExportManager::localSave(const QUrl &url, const QString &mimetype)
     QFile outputFile(url.toLocalFile());
 
     outputFile.open(QFile::WriteOnly);
-    if (!writeImage(&outputFile, mimetype.toLatin1())) {
+    if (!writeImage(&outputFile, suffix.toLatin1())) {
         Q_EMIT errorMessage(i18n("Cannot save screenshot. Error while writing file."));
         return false;
     }
     return true;
 }
 
-bool ExportManager::remoteSave(const QUrl &url, const QString &mimetype)
+bool ExportManager::remoteSave(const QUrl &url, const QString &suffix)
 {
     // Check if remote save directory exists
     const QUrl dirPath(url.adjusted(QUrl::RemoveFilename));
@@ -362,7 +372,7 @@ bool ExportManager::remoteSave(const QUrl &url, const QString &mimetype)
     QTemporaryFile tmpFile;
 
     if (tmpFile.open()) {
-        if (!writeImage(&tmpFile, mimetype.toLatin1())) {
+        if (!writeImage(&tmpFile, suffix.toLatin1())) {
             Q_EMIT errorMessage(i18n("Cannot save screenshot. Error while writing temporary local file."));
             return false;
         }
@@ -383,32 +393,32 @@ bool ExportManager::remoteSave(const QUrl &url, const QString &mimetype)
 QUrl ExportManager::tempSave()
 {
     // if we already have a temp file saved, use that
-    if (mTempFile.isValid()) {
-        if (QFile(mTempFile.toLocalFile()).exists()) {
-            return mTempFile;
+    if (m_tempFile.isValid()) {
+        if (QFile(m_tempFile.toLocalFile()).exists()) {
+            return m_tempFile;
         }
     }
 
-    if (!mTempDir) {
-        mTempDir = new QTemporaryDir(QDir::tempPath() + QDir::separator() + QStringLiteral("Spectacle.XXXXXX"));
+    if (!m_tempDir) {
+        m_tempDir = new QTemporaryDir(QDir::tempPath() + QDir::separator() + QStringLiteral("Spectacle.XXXXXX"));
     }
-    if (mTempDir && mTempDir->isValid()) {
+    if (m_tempDir && m_tempDir->isValid()) {
         // create the temporary file itself with normal file name and also unique one for this session
         // supports the use-case of creating multiple screenshots in a row
         // and exporting them to the same destination e.g. via clipboard,
         // where the temp file name is used as filename suggestion
-        const QString baseFileName = mTempDir->path() + QLatin1Char('/') + QUrl::fromLocalFile(makeAutosaveFilename()).fileName();
+        const QString baseFileName = m_tempDir->path() + QLatin1Char('/') + QUrl::fromLocalFile(formattedFilename()).fileName();
 
-        QString mimetype = makeSaveMimetype(QUrl(baseFileName));
-        const QString fileName = autoIncrementFilename(baseFileName, mimetype, &ExportManager::isTempFileAlreadyUsed);
+        QString suffix = imageFileSuffix(QUrl(baseFileName));
+        const QString fileName = autoIncrementFilename(baseFileName, suffix, &ExportManager::isTempFileAlreadyUsed);
         QFile tmpFile(fileName);
         if (tmpFile.open(QFile::WriteOnly)) {
-            if (writeImage(&tmpFile, mimetype.toLatin1())) {
-                mTempFile = QUrl::fromLocalFile(tmpFile.fileName());
+            if (writeImage(&tmpFile, suffix.toLatin1())) {
+                m_tempFile = QUrl::fromLocalFile(tmpFile.fileName());
                 // try to make sure 3rd-party which gets the url of the temporary file e.g. on export
                 // properly treats this as readonly, also hide from other users
                 tmpFile.setPermissions(QFile::ReadUser);
-                return mTempFile;
+                return m_tempFile;
             }
         }
     }
@@ -424,15 +434,15 @@ bool ExportManager::save(const QUrl &url)
         return false;
     }
 
-    const QString mimetype = makeSaveMimetype(url);
+    const QString suffix = imageFileSuffix(url);
     bool saveSucceded = false;
     if (url.isLocalFile()) {
-        saveSucceded = localSave(url, mimetype);
+        saveSucceded = localSave(url, suffix);
     } else {
-        saveSucceded = remoteSave(url, mimetype);
+        saveSucceded = remoteSave(url, suffix);
     }
     if (saveSucceded) {
-        mImageSavedNotInTemp = true;
+        m_imageSavedNotInTemp = true;
         KRecentDocument::add(url, QGuiApplication::desktopFileName());
     }
     return saveSucceded;
@@ -452,19 +462,19 @@ bool ExportManager::isFileExists(const QUrl &url) const
 
 bool ExportManager::isImageSavedNotInTemp() const
 {
-    return mImageSavedNotInTemp;
+    return m_imageSavedNotInTemp;
 }
 
 bool ExportManager::isTempFileAlreadyUsed(const QUrl &url) const
 {
-    return mUsedTempFileNames.contains(url);
+    return m_usedTempFileNames.contains(url);
 }
 
 // save slots
 
 void ExportManager::doSave(const QUrl &url, bool notify)
 {
-    if (mSavePixmap.isNull()) {
+    if (m_savePixmap.isNull()) {
         Q_EMIT errorMessage(i18n("Cannot save an empty screenshot image."));
         return;
     }
@@ -481,7 +491,7 @@ void ExportManager::doSave(const QUrl &url, bool notify)
     }
 }
 
-bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
+bool ExportManager::doSaveAs(bool notify)
 {
     QStringList supportedFilters;
 
@@ -495,11 +505,11 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
     // construct the file name
     const QString filenameExtension = Settings::self()->defaultSaveImageFormat().toLower();
     const QString mimetype = QMimeDatabase().mimeTypeForFile(QStringLiteral("~/fakefile.") + filenameExtension, QMimeDatabase::MatchExtension).name();
-    QFileDialog dialog(parentWindow);
+    QFileDialog dialog;
     dialog.setAcceptMode(QFileDialog::AcceptSave);
     dialog.setFileMode(QFileDialog::AnyFile);
     dialog.setDirectoryUrl(Settings::self()->lastSaveAsLocation().adjusted(QUrl::RemoveFilename));
-    dialog.selectFile(makeAutosaveFilename() + QStringLiteral(".") + filenameExtension);
+    dialog.selectFile(formattedFilename() + QStringLiteral(".") + filenameExtension);
     dialog.setDefaultSuffix(QStringLiteral(".") + filenameExtension);
     dialog.setMimeTypeFilters(supportedFilters);
     dialog.selectMimeTypeFilter(mimetype);
@@ -523,7 +533,7 @@ bool ExportManager::doSaveAs(QWidget *parentWindow, bool notify)
 
 void ExportManager::doSaveAndCopy(const QUrl &url)
 {
-    if (mSavePixmap.isNull()) {
+    if (m_savePixmap.isNull()) {
         Q_EMIT errorMessage(i18n("Cannot save an empty screenshot image."));
         return;
     }
@@ -542,7 +552,10 @@ void ExportManager::doSaveAndCopy(const QUrl &url)
 void ExportManager::doCopyToClipboard(bool notify)
 {
     auto data = new QMimeData();
-    data->setImageData(mSavePixmap.toImage());
+    data->setImageData(m_savePixmap.toImage());
+    // "x-kde-force-image-copy" is handled by Klipper.
+    // It ensures that the image is copied to Klipper even with the
+    // "Non-text selection: Never save in history" setting selected in Klipper.
     data->setData(QStringLiteral("x-kde-force-image-copy"), QByteArray());
     KSystemClipboard::instance()->setMimeData(data, QClipboard::Clipboard);
     Q_EMIT imageCopied();
@@ -554,7 +567,7 @@ void ExportManager::doCopyToClipboard(bool notify)
 void ExportManager::doCopyLocationToClipboard(bool notify)
 {
     QString localFile;
-    if (mImageSavedNotInTemp) {
+    if (m_imageSavedNotInTemp) {
         // The image has been saved (manually or automatically), we need to choose that file path
         localFile = Settings::self()->lastSaveLocation().toLocalFile();
     } else {
@@ -562,7 +575,9 @@ void ExportManager::doCopyLocationToClipboard(bool notify)
         localFile = ExportManager::instance()->tempSave().toLocalFile();
     }
 
-    QApplication::clipboard()->setText(localFile);
+    auto data = new QMimeData();
+    data->setText(localFile);
+    KSystemClipboard::instance()->setMimeData(data, QClipboard::Clipboard);
     Q_EMIT imageLocationCopied(QUrl::fromLocalFile(localFile));
     if (notify) {
         Q_EMIT forceNotify(QUrl::fromLocalFile(localFile));
@@ -575,20 +590,18 @@ void ExportManager::doPrint(QPrinter *printer)
 
     if (!(painter.begin(printer))) {
         Q_EMIT errorMessage(i18n("Printing failed. The printer failed to initialize."));
-        delete printer;
         return;
     }
 
+    painter.setRenderHint(QPainter::LosslessImageRendering);
+
     QRect devRect(0, 0, printer->width(), printer->height());
-    QPixmap pixmap = mSavePixmap.scaled(devRect.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
+    QPixmap pixmap = m_savePixmap.scaled(devRect.size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
     QRect srcRect = pixmap.rect();
     srcRect.moveCenter(devRect.center());
 
     painter.drawPixmap(srcRect.topLeft(), pixmap);
     painter.end();
-
-    delete printer;
-    return;
 }
 
 const QMap<QString, KLocalizedString> ExportManager::filenamePlaceholders{
