@@ -5,8 +5,7 @@
 
 #pragma once
 
-#include <SpectacleCommon.h>
-
+#include "settings.h"
 #include <KLocalizedString>
 #include <QDateTime>
 class QIODevice;
@@ -38,20 +37,30 @@ private:
 
 public:
     QString defaultSaveLocation() const;
+    QString defaultVideoSaveLocation() const;
     bool isFileExists(const QUrl &url) const;
     bool isImageSavedNotInTemp() const;
     void setPixmap(const QPixmap &pixmap);
     QPixmap pixmap() const;
     void updatePixmapTimestamp();
     void setTimestamp(const QDateTime &timestamp);
+
+    /**
+     * The title used to fill the window title template in formatted file names.
+     */
     QString windowTitle() const;
-    Spectacle::CaptureMode captureMode() const;
-    void setCaptureMode(Spectacle::CaptureMode theCaptureMode);
-    QString formatFilename(const QString &nameTemplate);
+
+    /**
+     * Returns a formatted filename using a template string.
+     */
+    QString formattedFilename(const QString &nameTemplate = Settings::saveFilenameFormat()) const;
+
+    QString suggestedVideoFilename(const QString &extension) const;
 
     static const QMap<QString, KLocalizedString> filenamePlaceholders;
 
 Q_SIGNALS:
+    void pixmapChanged();
 
     void errorMessage(const QString &str);
     void imageSaved(const QUrl &savedAt);
@@ -62,35 +71,33 @@ Q_SIGNALS:
 
 public Q_SLOTS:
 
-    QUrl getAutosaveFilename();
+    QUrl getAutosaveFilename() const;
     QUrl tempSave();
 
     void setWindowTitle(const QString &windowTitle);
     void doSave(const QUrl &url = QUrl(), bool notify = false);
-    bool doSaveAs(QWidget *parentWindow = nullptr, bool notify = false);
+    bool doSaveAs(bool notify = false);
     void doSaveAndCopy(const QUrl &url = QUrl());
     void doCopyToClipboard(bool notify = false);
     void doCopyLocationToClipboard(bool notify = false);
     void doPrint(QPrinter *printer);
 
 private:
-    QString truncatedFilename(const QString &filename);
-    QString makeAutosaveFilename();
+    QString truncatedFilename(const QString &filename) const;
     using FileNameAlreadyUsedCheck = bool (ExportManager::*)(const QUrl &) const;
-    QString autoIncrementFilename(const QString &baseName, const QString &extension, FileNameAlreadyUsedCheck isFileNameUsed);
-    QString makeSaveMimetype(const QUrl &url);
-    bool writeImage(QIODevice *device, const QByteArray &format);
+    QString autoIncrementFilename(const QString &baseName, const QString &extension, FileNameAlreadyUsedCheck isFileNameUsed) const;
+    QString imageFileSuffix(const QUrl &url) const;
+    bool writeImage(QIODevice *device, const QByteArray &suffix);
     bool save(const QUrl &url);
-    bool localSave(const QUrl &url, const QString &mimetype);
-    bool remoteSave(const QUrl &url, const QString &mimetype);
+    bool localSave(const QUrl &url, const QString &suffix);
+    bool remoteSave(const QUrl &url, const QString &suffix);
     bool isTempFileAlreadyUsed(const QUrl &url) const;
 
-    bool mImageSavedNotInTemp;
-    QPixmap mSavePixmap;
-    QDateTime mPixmapTimestamp;
-    QUrl mTempFile;
-    QTemporaryDir *mTempDir = nullptr;
-    QList<QUrl> mUsedTempFileNames;
-    QString mWindowTitle;
-    Spectacle::CaptureMode mCaptureMode{Spectacle::CaptureMode::AllScreens};
+    bool m_imageSavedNotInTemp;
+    QPixmap m_savePixmap;
+    QDateTime m_pixmapTimestamp;
+    QUrl m_tempFile;
+    QTemporaryDir *m_tempDir = nullptr;
+    QList<QUrl> m_usedTempFileNames;
+    QString m_windowTitle;
 };
