@@ -6,15 +6,11 @@
  */
 
 #include "Config.h"
-#include "SpectacleCommon.h"
 #include "SpectacleCore.h"
+#include "CommandLineOptions.h"
 #include "SpectacleDBusAdapter.h"
 #include "settings.h"
 
-#ifdef KIMAGEANNOTATOR_CAN_LOAD_TRANSLATIONS
-#include <kImageAnnotator/KImageAnnotator.h>
-#endif
-
 #include <QApplication>
 #include <QCommandLineParser>
 #include <QDBusConnection>
@@ -45,19 +41,21 @@ int main(int argc, char **argv)
                          i18n("(C) 2015 Boudhayan Gupta"));
     aboutData.addAuthor(QStringLiteral("Boudhayan Gupta"), QString(), QStringLiteral("bgupta@kde.org"));
     aboutData.addAuthor(QStringLiteral("David Redondo"), QString(), QStringLiteral("kde@david-redondo.de"));
+    aboutData.addAuthor(QStringLiteral("Noah Davis"), QString(), QStringLiteral("noahadvs@gmail.com"));
     aboutData.setTranslator(i18nc("NAME OF TRANSLATORS", "Your names"), i18nc("EMAIL OF TRANSLATORS", "Your emails"));
     KAboutData::setApplicationData(aboutData);
     app.setWindowIcon(QIcon::fromTheme(QStringLiteral("spectacle")));
 
-    SpectacleCore lCore;
-
-    QCommandLineParser lCmdLineParser;
-    aboutData.setupCommandLine(&lCmdLineParser);
-    lCore.populateCommandLineParser(&lCmdLineParser);
+    QCommandLineParser commandLineParser;
+    aboutData.setupCommandLine(&commandLineParser);
+    commandLineParser.addOptions(CommandLineOptions::self()->allOptions);
 
     // first parsing for help-about
-    lCmdLineParser.process(app.arguments());
-    aboutData.processCommandLine(&lCmdLineParser);
+    commandLineParser.process(app.arguments());
+    aboutData.processCommandLine(&commandLineParser);
+
+    // Prevent session manager from restoring the app on start up.
+    // https://bugs.kde.org/show_bug.cgi?id=430411
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QGuiApplication::setFallbackSessionManagementEnabled(false);
 #endif
@@ -67,47 +65,43 @@ int main(int argc, char **argv)
     QObject::connect(&app, &QGuiApplication::commitDataRequest, disableSessionManagement);
     QObject::connect(&app, &QGuiApplication::saveStateRequest, disableSessionManagement);
 
-    // and new-instance
-    if (lCmdLineParser.isSet(QStringLiteral("new-instance"))) {
-        lCore.init();
+    // If the new instance command line option has been specified,
+    // use this alternative path for executing Spectacle.
+    if (commandLineParser.isSet(CommandLineOptions::self()->newInstance)) {
+        SpectacleCore spectacleCore;
 
-        QObject::connect(&lCore, &SpectacleCore::allDone, &app, &QCoreApplication::quit, Qt::QueuedConnection);
+        QObject::connect(qApp, &QApplication::aboutToQuit, Settings::self(), &Settings::save);
+        QObject::connect(&spectacleCore, &SpectacleCore::allDone, &app, &QCoreApplication::quit, Qt::QueuedConnection);
 
         // fire it up
-        lCore.onActivateRequested(app.arguments(), QStringLiteral());
+        spectacleCore.activate(app.arguments(), QLatin1String());
 
         return app.exec();
     }
 
-    // Ensure that we only launch a new instance if we need to
-    // If there is already an instance running, we will quit here
-    // and activateRequested signal is triggered
-    // For some reason this does not work properly if behind an if
-    KDBusService service(KDBusService::Unique, &lCore);
+    // With the StartupOption::Unique flag, this process will exit during the construction of
+    // KDBusService if Spectacle has already been registered.
+    // This object does not need a parent since it will be deleted when it falls out of scope.
+    KDBusService service(KDBusService::Unique);
 
-#ifdef KIMAGEANNOTATOR_CAN_LOAD_TRANSLATIONS
-    kImageAnnotator::loadTranslations();
-#endif
+    SpectacleCore spectacleCore;
 
-    // Delay initialisation after we now we are in the single instance or new-instance was passed, to avoid doing it each time spectacle executable is called
-    lCore.init();
+    QObject::connect(&service, &KDBusService::activateRequested, &spectacleCore, &SpectacleCore::activate);
 
-    // set up the KDBusService activateRequested slot
-    QObject::connect(&service, &KDBusService::activateRequested, &lCore, &SpectacleCore::onActivateRequested);
-    QObject::connect(&lCore, &SpectacleCore::allDone, &app, &QCoreApplication::quit, Qt::QueuedConnection);
-    QObject::connect(qApp, &QApplication::aboutToQuit, Settings::self(), &Settings::save);
+    QObject::connect(&app, &QCoreApplication::aboutToQuit, Settings::self(), &Settings::save);
+    QObject::connect(&spectacleCore, &SpectacleCore::allDone, &app, &QCoreApplication::quit, Qt::QueuedConnection);
 
     // create the dbus connections
-    SpectacleDBusAdapter *lDBusAdapter = new SpectacleDBusAdapter(&lCore);
-    QObject::connect(&lCore, &SpectacleCore::grabFailed, lDBusAdapter, &SpectacleDBusAdapter::ScreenshotFailed);
-    QObject::connect(ExportManager::instance(), &ExportManager::imageSaved, &lCore, [&](const QUrl &savedAt) {
-        lDBusAdapter->ScreenshotTaken(savedAt.toLocalFile());
+    SpectacleDBusAdapter *lDBusAdapter = new SpectacleDBusAdapter(&spectacleCore);
+    QObject::connect(&spectacleCore, &SpectacleCore::grabFailed, lDBusAdapter, &SpectacleDBusAdapter::ScreenshotFailed);
+    QObject::connect(ExportManager::instance(), &ExportManager::imageSaved, &spectacleCore, [&](const QUrl &savedAt) {
+        Q_EMIT lDBusAdapter->ScreenshotTaken(savedAt.toLocalFile());
     });
-    QDBusConnection::sessionBus().registerObject(QStringLiteral("/"), &lCore);
+    QDBusConnection::sessionBus().registerObject(QStringLiteral("/"), &spectacleCore);
     QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.Spectacle"));
 
     // fire it up
-    lCore.onActivateRequested(app.arguments(), QStringLiteral());
+    spectacleCore.activate(app.arguments(), QLatin1String());
 
     return app.exec();
 }
