@@ -15,6 +15,7 @@
 #include <QFutureWatcher>
 #include <QGuiApplication>
 #include <QPixmap>
+#include <QScreen>
 #include <QtConcurrent>
 
 #include <errno.h>
@@ -43,7 +44,7 @@ static const QString s_screenShotService = QStringLiteral("org.kde.KWin.ScreenSh
 static const QString s_screenShotObjectPath = QStringLiteral("/org/kde/KWin/ScreenShot2");
 static const QString s_screenShotInterface = QStringLiteral("org.kde.KWin.ScreenShot2");
 
-template <typename ...ArgType>
+template<typename... ArgType>
 ScreenShotSource2::ScreenShotSource2(const QString &methodName, ArgType... arguments)
 {
     // Do not set the O_NONBLOCK flag. Code that reads data from the pipe assumes
@@ -55,10 +56,7 @@ ScreenShotSource2::ScreenShotSource2(const QString &methodName, ArgType... argum
         return;
     }
 
-    QDBusMessage message = QDBusMessage::createMethodCall(s_screenShotService,
-                                                          s_screenShotObjectPath,
-                                                          s_screenShotInterface,
-                                                          methodName);
+    QDBusMessage message = QDBusMessage::createMethodCall(s_screenShotService, s_screenShotObjectPath, s_screenShotInterface, methodName);
 
     QVariantList dbusArguments{arguments...};
     dbusArguments.append(QVariant::fromValue(QDBusUnixFileDescriptor(pipeFds[1])));
@@ -164,26 +162,33 @@ void ScreenShotSource2::handleMetaDataReceived(const QVariantMap &metadata)
     m_pipeFileDescriptor = -1;
 }
 
-ScreenShotSourceArea2::ScreenShotSourceArea2(const QRect &area,
-                                             PlatformKWinWayland2::ScreenShotFlags flags)
+ScreenShotSourceArea2::ScreenShotSourceArea2(const QRect &area, PlatformKWinWayland2::ScreenShotFlags flags)
     : ScreenShotSource2(QStringLiteral("CaptureArea"),
-                        qint32(area.x()), qint32(area.y()),
-                        quint32(area.width()), quint32(area.height()),
+                        qint32(area.x()),
+                        qint32(area.y()),
+                        quint32(area.width()),
+                        quint32(area.height()),
                         screenShotFlagsToVardict(flags))
 {
 }
 
-ScreenShotSourceInteractive2::ScreenShotSourceInteractive2(PlatformKWinWayland2::InteractiveKind kind,
-                                                           PlatformKWinWayland2::ScreenShotFlags flags)
-    : ScreenShotSource2(QStringLiteral("CaptureInteractive"),
-                        quint32(kind), screenShotFlagsToVardict(flags))
+ScreenShotSourceInteractive2::ScreenShotSourceInteractive2(PlatformKWinWayland2::InteractiveKind kind, PlatformKWinWayland2::ScreenShotFlags flags)
+    : ScreenShotSource2(QStringLiteral("CaptureInteractive"), quint32(kind), screenShotFlagsToVardict(flags))
 {
 }
 
-ScreenShotSourceScreen2::ScreenShotSourceScreen2(const QScreen *screen,
-                                                 PlatformKWinWayland2::ScreenShotFlags flags)
-    : ScreenShotSource2(QStringLiteral("CaptureScreen"),
-                        screen->name(), screenShotFlagsToVardict(flags))
+ScreenShotSourceScreen2::ScreenShotSourceScreen2(const QScreen *screen, PlatformKWinWayland2::ScreenShotFlags flags)
+    : ScreenShotSource2(QStringLiteral("CaptureScreen"), screen->name(), screenShotFlagsToVardict(flags))
+{
+}
+
+ScreenShotSourceActiveWindow2::ScreenShotSourceActiveWindow2(PlatformKWinWayland2::ScreenShotFlags flags)
+    : ScreenShotSource2(QStringLiteral("CaptureActiveWindow"), screenShotFlagsToVardict(flags))
+{
+}
+
+ScreenShotSourceActiveScreen2::ScreenShotSourceActiveScreen2(PlatformKWinWayland2::ScreenShotFlags flags)
+    : ScreenShotSource2(QStringLiteral("CaptureActiveScreen"), screenShotFlagsToVardict(flags))
 {
 }
 
@@ -193,10 +198,8 @@ ScreenShotSourceMeta2::ScreenShotSourceMeta2(const QVector<ScreenShotSource2 *>
     for (ScreenShotSource2 *source : sources) {
         source->setParent(this);
 
-        connect(source, &ScreenShotSource2::finished,
-                this, &ScreenShotSourceMeta2::handleSourceFinished);
-        connect(source, &ScreenShotSource2::errorOccurred,
-                this, &ScreenShotSourceMeta2::handleSourceErrorOccurred);
+        connect(source, &ScreenShotSource2::finished, this, &ScreenShotSourceMeta2::handleSourceFinished);
+        connect(source, &ScreenShotSource2::errorOccurred, this, &ScreenShotSourceMeta2::handleSourceErrorOccurred);
     }
 }
 
@@ -212,7 +215,7 @@ void ScreenShotSourceMeta2::handleSourceFinished()
     QVector<QImage> results;
     results.reserve(m_sources.count());
 
-    for (const ScreenShotSource2 *source : qAsConst(m_sources)) {
+    for (const ScreenShotSource2 *source : std::as_const(m_sources)) {
         results.append(source->result());
     }
 
@@ -236,26 +239,43 @@ std::unique_ptr<PlatformKWinWayland2> PlatformKWinWayland2::create()
 PlatformKWinWayland2::PlatformKWinWayland2(QObject *parent)
     : Platform(parent)
 {
-}
+    auto message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.KWin.ScreenShot2"),
+                                                  QStringLiteral("/org/kde/KWin/ScreenShot2"),
+                                                  QStringLiteral("org.freedesktop.DBus.Properties"),
+                                                  QStringLiteral("Get"));
+    message.setArguments({QStringLiteral("org.kde.KWin.ScreenShot2"), QStringLiteral("Version")});
 
-QString PlatformKWinWayland2::platformName() const
-{
-    return QStringLiteral("PlatformKWinWayland2");
+    const QDBusMessage reply = QDBusConnection::sessionBus().call(message);
+    if (reply.type() == QDBusMessage::ReplyMessage) {
+        m_apiVersion = reply.arguments().constFirst().value<QDBusVariant>().variant().toUInt();
+    }
+
+    updateSupportedGrabModes();
+    connect(qGuiApp, &QGuiApplication::screenAdded, this, &PlatformKWinWayland2::updateSupportedGrabModes);
+    connect(qGuiApp, &QGuiApplication::screenRemoved, this, &PlatformKWinWayland2::updateSupportedGrabModes);
 }
 
 Platform::GrabModes PlatformKWinWayland2::supportedGrabModes() const
 {
-    Platform::GrabModes supportedModes = GrabMode::AllScreens |
-                                         GrabMode::WindowUnderCursor |
-                                         GrabMode::PerScreenImageNative |
-                                         GrabMode::AllScreensScaled;
+    return m_grabModes;
+}
 
-    const QList<QScreen *> screens = QGuiApplication::screens();
-    if (screens.count() > 1) {
-        supportedModes |= GrabMode::CurrentScreen;
+void PlatformKWinWayland2::updateSupportedGrabModes()
+{
+    Platform::GrabModes grabModes = GrabMode::AllScreens | GrabMode::WindowUnderCursor | GrabMode::PerScreenImageNative;
+
+    if (m_apiVersion >= 2) {
+        grabModes |= GrabMode::ActiveWindow;
     }
 
-    return supportedModes;
+    if (QGuiApplication::screens().count() > 1) {
+        grabModes |= GrabMode::CurrentScreen | GrabMode::AllScreensScaled;
+    }
+
+    if (m_grabModes != grabModes) {
+        m_grabModes = grabModes;
+        Q_EMIT supportedGrabModesChanged();
+    }
 }
 
 Platform::ShutterModes PlatformKWinWayland2::supportedShutterModes() const
@@ -274,8 +294,7 @@ static QRect workArea()
     return std::accumulate(screens.begin(), screens.end(), QRect(), accumulateFunc);
 }
 
-void PlatformKWinWayland2::doGrab(ShutterMode, GrabMode theGrabMode,
-                                  bool theIncludePointer, bool theIncludeDecorations)
+void PlatformKWinWayland2::doGrab(ShutterMode, GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations)
 {
     ScreenShotFlags flags = ScreenShotFlag::NativeSize;
 
@@ -291,8 +310,16 @@ void PlatformKWinWayland2::doGrab(ShutterMode, GrabMode theGrabMode,
         takeScreenShotArea(workArea(), flags);
         break;
     case GrabMode::CurrentScreen:
-        takeScreenShotInteractive(InteractiveKind::Screen, flags);
+        if (m_apiVersion >= 2) {
+            takeScreenShotActiveScreen(flags);
+        } else {
+            takeScreenShotInteractive(InteractiveKind::Screen, flags);
+        }
         break;
+    case GrabMode::ActiveWindow:
+        takeScreenShotActiveWindow(flags);
+        break;
+    case GrabMode::TransientWithParent:
     case GrabMode::WindowUnderCursor:
         takeScreenShotInteractive(InteractiveKind::Window, flags);
         break;
@@ -302,10 +329,7 @@ void PlatformKWinWayland2::doGrab(ShutterMode, GrabMode theGrabMode,
     case GrabMode::PerScreenImageNative:
         takeScreenShotScreens(QGuiApplication::screens(), flags);
         break;
-
-    case GrabMode::InvalidChoice:
-    case GrabMode::TransientWithParent:
-    case GrabMode::ActiveWindow:
+    case GrabMode::NoGrabModes:
         Q_EMIT newScreenshotFailed();
         break;
     }
@@ -327,7 +351,17 @@ void PlatformKWinWayland2::trackSource(ScreenShotSourceMeta2 *source)
 {
     connect(source, &ScreenShotSourceMeta2::finished, this, [this, source](const QVector<QImage> &images) {
         source->deleteLater();
-        Q_EMIT newScreensScreenshotTaken(images);
+        QVector<ScreenImage> screenImages;
+        const auto &screens = qGuiApp->screens();
+        if (images.length() != screens.length()) {
+            qWarning() << "ERROR: number of screens does not match number of images, expected:" << images.length() << "actual:" << screens.length();
+            Q_EMIT newScreenshotFailed();
+            return;
+        }
+        for (int i = 0; i < screens.length(); ++i) {
+            screenImages.append({screens.at(i), images.at(i), static_cast<qreal>(images.at(i).width()) / screens.at(i)->geometry().width()});
+        }
+        Q_EMIT newScreensScreenshotTaken(screenImages);
     });
     connect(source, &ScreenShotSourceMeta2::errorOccurred, this, [this, source]() {
         source->deleteLater();
@@ -340,14 +374,22 @@ void PlatformKWinWayland2::takeScreenShotArea(const QRect &area, ScreenShotFlags
     trackSource(new ScreenShotSourceArea2(area, flags));
 }
 
-void PlatformKWinWayland2::takeScreenShotInteractive(InteractiveKind kind,
-                                                     ScreenShotFlags flags)
+void PlatformKWinWayland2::takeScreenShotInteractive(InteractiveKind kind, ScreenShotFlags flags)
 {
     trackSource(new ScreenShotSourceInteractive2(kind, flags));
 }
 
-void PlatformKWinWayland2::takeScreenShotScreens(const QList<QScreen *> &screens,
-                                                 ScreenShotFlags flags)
+void PlatformKWinWayland2::takeScreenShotActiveWindow(ScreenShotFlags flags)
+{
+    trackSource(new ScreenShotSourceActiveWindow2(flags));
+}
+
+void PlatformKWinWayland2::takeScreenShotActiveScreen(ScreenShotFlags flags)
+{
+    trackSource(new ScreenShotSourceActiveScreen2(flags));
+}
+
+void PlatformKWinWayland2::takeScreenShotScreens(const QList<QScreen *> &screens, ScreenShotFlags flags)
 {
     QVector<ScreenShotSource2 *> sources;
     sources.reserve(screens.count());
