@@ -6,6 +6,7 @@
 
 #include "PlatformNull.h"
 
+#include <QDebug>
 #include <QPixmap>
 
 /* -- Null Platform ---------------------------------------------------------------------------- */
@@ -15,11 +16,6 @@ PlatformNull::PlatformNull(QObject *parent)
 {
 }
 
-QString PlatformNull::platformName() const
-{
-    return QStringLiteral("Null");
-}
-
 Platform::GrabModes PlatformNull::supportedGrabModes() const
 {
     return {GrabMode::AllScreens | GrabMode::CurrentScreen | GrabMode::ActiveWindow | GrabMode::WindowUnderCursor | GrabMode::TransientWithParent
@@ -39,3 +35,32 @@ void PlatformNull::doGrab(ShutterMode theShutterMode, GrabMode theGrabMode, bool
     Q_UNUSED(theIncludeDecorations)
     Q_EMIT newScreenshotTaken(QPixmap());
 }
+
+VideoPlatformNull::VideoPlatformNull(QObject *parent)
+    : VideoPlatform(parent)
+{
+}
+
+VideoPlatform::RecordingModes VideoPlatformNull::supportedRecordingModes() const
+{
+    return {};
+}
+
+void VideoPlatformNull::startRecording(const QString &path, RecordingMode mode, const RecordingOption &, bool withPointer)
+{
+    setRecording(true);
+    m_path = path;
+    qDebug() << "start recording" << mode << "pointer:" << withPointer << path;
+}
+
+void VideoPlatformNull::finishRecording()
+{
+    setRecording(false);
+    qDebug() << "finish recording" << m_path;
+    Q_EMIT recordingSaved(m_path);
+}
+
+QString VideoPlatformNull::extension() const
+{
+    return QStringLiteral("mp4");
+}
