@@ -35,6 +35,7 @@
 
 #include <KWindowInfo>
 #include <KWindowSystem>
+#include <KX11Extras>
 
 #include <memory>
 
@@ -95,7 +96,7 @@ public:
                 {
                     auto lSecondEvent = static_cast<xcb_button_release_event_t *>(theMessage);
                     if (lSecondEvent->detail == 1) {
-                        QTimer::singleShot(0, [this]() {
+                        QTimer::singleShot(0, nullptr, [this]() {
                             mPlatformPtr->doGrabNow(mGrabMode, mIncludePointer, mIncludeDecorations);
                         });
                     } else if (lSecondEvent->detail == 2 || lSecondEvent->detail == 3) {
@@ -104,7 +105,7 @@ public:
                     } else if (lSecondEvent->detail < 4) {
                         Q_EMIT mPlatformPtr->newScreenshotFailed();
                     } else {
-                        QTimer::singleShot(0, [this]() {
+                        QTimer::singleShot(0, nullptr, [this]() {
                             mPlatformPtr->doGrabOnClick(mGrabMode, mIncludePointer, mIncludeDecorations);
                         });
                     }
@@ -129,28 +130,38 @@ private:
 
 PlatformXcb::PlatformXcb(QObject *theParent)
     : Platform(theParent)
-    , mNativeEventFilter(new OnClickEventFilter(this))
+    , m_nativeEventFilter(new OnClickEventFilter(this))
 {
+    updateSupportedGrabModes();
+    connect(qGuiApp, &QGuiApplication::screenAdded, this, &PlatformXcb::updateSupportedGrabModes);
+    connect(qGuiApp, &QGuiApplication::screenRemoved, this, &PlatformXcb::updateSupportedGrabModes);
 }
 
 PlatformXcb::~PlatformXcb()
 {
-    delete mNativeEventFilter;
-}
-
-QString PlatformXcb::platformName() const
-{
-    return QStringLiteral("Xcb");
 }
 
 Platform::GrabModes PlatformXcb::supportedGrabModes() const
 {
-    Platform::GrabModes lSupportedModes(
-        {GrabMode::AllScreens, GrabMode::ActiveWindow, GrabMode::WindowUnderCursor, GrabMode::TransientWithParent, GrabMode::PerScreenImageNative});
+    return m_grabModes;
+}
+
+void PlatformXcb::updateSupportedGrabModes()
+{
+    Platform::GrabModes grabModes = {
+        GrabMode::AllScreens, GrabMode::ActiveWindow,
+        GrabMode::WindowUnderCursor, GrabMode::TransientWithParent,
+        GrabMode::PerScreenImageNative
+    };
+
     if (QApplication::screens().count() > 1) {
-        lSupportedModes |= Platform::GrabMode::CurrentScreen;
+        grabModes |= Platform::GrabMode::CurrentScreen;
+    }
+
+    if (m_grabModes != grabModes) {
+        m_grabModes = grabModes;
+        Q_EMIT supportedGrabModesChanged();
     }
-    return lSupportedModes;
 }
 
 Platform::ShutterModes PlatformXcb::supportedShutterModes() const
@@ -176,7 +187,7 @@ void PlatformXcb::doGrab(ShutterMode theShutterMode, GrabMode theGrabMode, bool
 
 void PlatformXcb::updateWindowTitle(xcb_window_t theWindow)
 {
-    auto lTitle = KWindowSystem::readNameProperty(theWindow, XA_WM_NAME);
+    auto lTitle = KX11Extras::readNameProperty(theWindow, XA_WM_NAME);
     Q_EMIT windowTitleChanged(lTitle);
 }
 
@@ -532,7 +543,7 @@ void PlatformXcb::grabApplicationWindow(xcb_window_t theWindow, bool theIncludeP
 
 void PlatformXcb::grabActiveWindow(bool theIncludePointer, bool theIncludeDecorations)
 {
-    auto lActiveWindow = KWindowSystem::activeWindow();
+    auto lActiveWindow = KX11Extras::activeWindow();
     updateWindowTitle(lActiveWindow);
 
     // if KWin is available, use the KWin DBus interfaces
@@ -622,7 +633,7 @@ void PlatformXcb::grabTransientWithParent(bool theIncludePointer, bool theInclud
     // All parents are known now, find other transient children.
     // Assume that the lowest window is behind everything else, then if a new
     // transient window is discovered, its children can then also be found.
-    auto lWinList = KWindowSystem::stackingOrder();
+    auto lWinList = KX11Extras::stackingOrder();
     for (auto lWinId : lWinList) {
         QRect lWinRect;
         auto lParentWindow = getTransientWindowParent(lWinId, lWinRect, theIncludeDecorations);
@@ -675,6 +686,10 @@ void PlatformXcb::grabTransientWithParent(bool theIncludePointer, bool theInclud
 
 void PlatformXcb::doGrabNow(GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations)
 {
+    if (theGrabMode & ~(ActiveWindow | WindowUnderCursor | TransientWithParent)) {
+        // Notify that window title is empty since we are not picking a window.
+        Q_EMIT windowTitleChanged();
+    }
     switch (theGrabMode) {
     case GrabMode::AllScreens:
     case GrabMode::AllScreensScaled:
@@ -684,13 +699,13 @@ void PlatformXcb::doGrabNow(GrabMode theGrabMode, bool theIncludePointer, bool t
         auto lPixmap = getToplevelPixmap(QRect(), theIncludePointer);
         // break thePixmap into list of images
         const auto screens = QGuiApplication::screens();
-        QVector<QImage> images;
+        QVector<ScreenImage> screenImages;
         for (const auto screen : screens) {
             QRect geom = screen->geometry();
             geom.setSize(screen->size() * screen->devicePixelRatio());
-            images << lPixmap.copy(geom).toImage();
+            screenImages.append({screen, lPixmap.copy(geom).toImage(), screen->devicePixelRatio()});
         }
-        Q_EMIT newScreensScreenshotTaken(images);
+        Q_EMIT newScreensScreenshotTaken(screenImages);
         break;
     }
     case GrabMode::CurrentScreen:
@@ -705,7 +720,7 @@ void PlatformXcb::doGrabNow(GrabMode theGrabMode, bool theIncludePointer, bool t
     case GrabMode::TransientWithParent:
         grabTransientWithParent(theIncludePointer, theIncludeDecorations);
         break;
-    case GrabMode::InvalidChoice:
+    case GrabMode::NoGrabModes:
         Q_EMIT newScreenshotFailed();
     }
 }
@@ -752,9 +767,9 @@ void PlatformXcb::doGrabOnClick(GrabMode theGrabMode, bool theIncludePointer, bo
     }
 
     // fix things if our pointer grab causes a lockup and install our event filter
-    mNativeEventFilter->setCaptureOptions(theGrabMode, theIncludePointer, theIncludeDecorations);
+    m_nativeEventFilter->setCaptureOptions(theGrabMode, theIncludePointer, theIncludeDecorations);
     xcb_allow_events(QX11Info::connection(), XCB_ALLOW_SYNC_POINTER, XCB_TIME_CURRENT_TIME);
-    qApp->installNativeEventFilter(mNativeEventFilter);
+    qApp->installNativeEventFilter(m_nativeEventFilter.get());
 
     // done. clean stuff up
     xcb_cursor_context_free(lXcbCursorCtx);
