@@ -13,31 +13,27 @@
 
 #include <QPixmap>
 
-class PlatformXcb final: public Platform
+class PlatformXcb final : public Platform
 {
     Q_OBJECT
 
-    public:
-
+public:
     explicit PlatformXcb(QObject *parent = nullptr);
-    virtual ~PlatformXcb();
+    ~PlatformXcb() override;
 
-    QString platformName() const override final;
     GrabModes supportedGrabModes() const override final;
     ShutterModes supportedShutterModes() const override final;
 
-    public Q_SLOTS:
-
+public Q_SLOTS:
     void doGrab(Platform::ShutterMode theShutterMode, Platform::GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations) override final;
 
-    private Q_SLOTS:
-
+private Q_SLOTS:
+    void updateSupportedGrabModes();
     void handleKWinScreenshotReply(quint64 theDrawable);
     void doGrabNow(Platform::GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations);
     void doGrabOnClick(Platform::GrabMode theGrabMode, bool theIncludePointer, bool theIncludeDecorations);
 
-    private:
-
+private:
     inline void updateWindowTitle(xcb_window_t theWindow);
     bool isKWinAvailable();
     QPoint getCursorPosition();
@@ -60,5 +56,7 @@ class PlatformXcb final: public Platform
 
     // on-click screenshot shutter support needs a native event filter in xcb
     class OnClickEventFilter;
-    OnClickEventFilter *mNativeEventFilter;
+    std::unique_ptr<OnClickEventFilter> m_nativeEventFilter;
+
+    GrabModes m_grabModes;
 };
